{-# LANGUAGE TypeApplications #-}
module Control.Addressbook.Query where

import Data.Text
       (Text)
import qualified Data.Text as T

import Conduit
import qualified Data.Conduit.Binary as CB
import qualified Data.Conduit.Combinators as C
import qualified Data.Conduit.Text as CT


import System.IO
       (stdout)

import Data.Maybe
       (fromMaybe)
import System.Environment
       (lookupEnv)
import System.FilePath
       ((</>))

import Control.Exception
       (catch)

-- XXX: The current Data.Trie implementation is much slower than Data.Set
query :: Text -> IO ()
query prefix = do
  datDir <- fromMaybe "./" <$> lookupEnv "HOME"
  catch @IOError (runResourceT $ runConduit $ filterAddresses datDir) $ \e -> print e
  where
    prefixLC = T.toLower prefix
    filterAddresses :: FilePath -> ConduitT () Void (ResourceT IO) ()
    filterAddresses dir =
      CB.sourceFile (dir </> ".addressbook.dat")
        .| CT.decode CT.utf8
        .| CT.lines
        .| C.filter (\v -> prefixLC `T.isPrefixOf` T.toLower v)
        .| CT.encode CT.utf8
        .| C.map (<> "\n")
        .| CB.sinkHandle stdout
